@@ -1219,9 +1219,8 @@ static struct connection *conn_backend_get(struct stream *s, struct server *srv,
 			session_add_conn(s->sess, conn, conn->target);
 		}
 		else {
-			ebmb_insert(&srv->per_thr[tid].avail_conns,
-			            &conn->hash_node->node,
-			            sizeof(conn->hash_node->hash));
+			eb64_insert(&srv->per_thr[tid].avail_conns,
+			            &conn->hash_node->node);
 		}
 	}
 	return conn;
@@ -1532,7 +1531,7 @@ skip_reuse:
 				return SF_ERR_RESOURCE;
 			}
 
-			srv_conn->hash_node->hash = hash;
+			srv_conn->hash_node->node.key = hash;
 		}
 	}
 
@@ -1700,7 +1699,7 @@ skip_reuse:
 			if (srv && reuse_mode == PR_O_REUSE_ALWS &&
 			    !(srv_conn->flags & CO_FL_PRIVATE) &&
 			    srv_conn->mux->avail_streams(srv_conn) > 0) {
-				ebmb_insert(&srv->per_thr[tid].avail_conns, &srv_conn->hash_node->node, sizeof(srv_conn->hash_node->hash));
+				eb64_insert(&srv->per_thr[tid].avail_conns, &srv_conn->hash_node->node);
 			}
 			else if (srv_conn->flags & CO_FL_PRIVATE ||
 			         (reuse_mode == PR_O_REUSE_SAFE &&
@@ -2319,7 +2318,6 @@ void back_handle_st_cer(struct stream *s)
 
 		/* only wait when we're retrying on the same server */
 		if ((si->state == SI_ST_ASS ||
-		     (s->be->lbprm.algo & BE_LB_KIND) != BE_LB_KIND_RR ||
 		     (s->be->srv_act <= 1)) && !reused) {
 			si->state = SI_ST_TAR;
 			si->exp = tick_add(now_ms, MS_TO_TICKS(delay));
