@@ -538,18 +538,24 @@ void pool_destroy_all()
 {
 	struct pool_head *entry, *back;
 
-	list_for_each_entry_safe(entry, back, &pools, list)
+	list_for_each_entry_safe(entry, back, &pools, list) {
+		/* there's only one occurrence of each pool in the list,
+		 * and we're existing instead of looping on the whole
+		 * list just to decrement users, force it to 1 here.
+		 */
+		entry->users = 1;
 		pool_destroy(entry);
+	}
 }
 
 /* This function dumps memory usage information into the trash buffer. */
 void dump_pools_to_trash()
 {
 	struct pool_head *entry;
-	unsigned long allocated, used;
+	unsigned long long allocated, used;
 	int nbpools;
 #ifdef CONFIG_HAP_POOLS
-	unsigned long cached_bytes = 0;
+	unsigned long long cached_bytes = 0;
 	uint cached = 0;
 #endif
 
@@ -560,15 +566,15 @@ void dump_pools_to_trash()
 		int i;
 		for (cached = i = 0; i < global.nbthread; i++)
 			cached += entry->cache[i].count;
-		cached_bytes += cached * entry->size;
+		cached_bytes += cached * (ullong)entry->size;
 #endif
-		chunk_appendf(&trash, "  - Pool %s (%u bytes) : %u allocated (%u bytes), %u used"
+		chunk_appendf(&trash, "  - Pool %s (%u bytes) : %u allocated (%llu bytes), %u used"
 #ifdef CONFIG_HAP_POOLS
 			      " (~%u by thread caches)"
 #endif
 			      ", needed_avg %u, %u failures, %u users, @%p%s\n",
 		              entry->name, entry->size, entry->allocated,
-		              entry->size * entry->allocated, entry->used,
+		              (ullong)entry->size * entry->allocated, entry->used,
 #ifdef CONFIG_HAP_POOLS
 		              cached,
 #endif
@@ -576,13 +582,13 @@ void dump_pools_to_trash()
 		              entry->users, entry,
 		              (entry->flags & MEM_F_SHARED) ? " [SHARED]" : "");
 
-		allocated += entry->allocated * entry->size;
-		used += entry->used * entry->size;
+		allocated += entry->allocated * (ullong)entry->size;
+		used += entry->used * (ullong)entry->size;
 		nbpools++;
 	}
-	chunk_appendf(&trash, "Total: %d pools, %lu bytes allocated, %lu used"
+	chunk_appendf(&trash, "Total: %d pools, %llu bytes allocated, %llu used"
 #ifdef CONFIG_HAP_POOLS
-		      " (~%lu by thread caches)"
+		      " (~%llu by thread caches)"
 #endif
 		      ".\n",
 	              nbpools, allocated, used
@@ -611,24 +617,24 @@ int pool_total_failures()
 }
 
 /* This function returns the total amount of memory allocated in pools (in bytes) */
-unsigned long pool_total_allocated()
+unsigned long long pool_total_allocated()
 {
 	struct pool_head *entry;
-	unsigned long allocated = 0;
+	unsigned long long allocated = 0;
 
 	list_for_each_entry(entry, &pools, list)
-		allocated += entry->allocated * entry->size;
+		allocated += entry->allocated * (ullong)entry->size;
 	return allocated;
 }
 
 /* This function returns the total amount of memory used in pools (in bytes) */
-unsigned long pool_total_used()
+unsigned long long pool_total_used()
 {
 	struct pool_head *entry;
-	unsigned long used = 0;
+	unsigned long long used = 0;
 
 	list_for_each_entry(entry, &pools, list)
-		used += entry->used * entry->size;
+		used += entry->used * (ullong)entry->size;
 	return used;
 }
 
